unit MatrixImage;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ImgList;

type
  TMatrixImage = class(TGraphicControl)
  private
    FPictures: array of array of TPicture;
    FPicturesIndex: array of array of Integer;
    FRow: Integer;
    FCol: Integer;
    FPictureHeight: Integer;
    FPictureWidth: Integer;
    FImageList: TCustomImageList;
    FStretch: Boolean;
    FCenter: Boolean;
    FTransparent: Boolean;
    FDrawing: Boolean;
    procedure PictureChanged(Sender: TObject);
    procedure SetCenter(Value: Boolean);
    procedure SetStretch(Value: Boolean);
    procedure SetTransparent(Value: Boolean);
    procedure SetRow(Value: Integer);
    procedure SetCol(Value: Integer);
    procedure SetPictureHeight(Value: Integer);
    procedure SetPictureWidth(Value: Integer);
    procedure SetImageList(const Value: TCustomImageList);
    procedure ResizePicturesMatrix(NewRow, NewCol: Integer);
    function PictureExist(RowNum, ColNum: Integer): Boolean;
  protected
    function DestRect(RowNum, ColNum: Integer): TRect;
    function DoPaletteChange(RowNum, ColNum: Integer): Boolean;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AssignPictureIndex(RowNum, ColNum, Index: Integer);
    procedure RemovePicture(RowNum, ColNum: Integer);
  published
    property Align;
    property Anchors;
    property Center: Boolean read FCenter write SetCenter default False;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Stretch: Boolean read FStretch write SetStretch default False;
    property Transparent: Boolean read FTransparent write SetTransparent default False;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property Row: Integer read FRow write SetRow default 1;
    property Col: Integer read FCol write SetCol default 1;
    property PictureHeight: Integer read FPictureHeight write SetPictureHeight default 32;
    property PictureWidth: Integer read FPictureWidth write SetPictureWidth default 32;
    property ImageList: TCustomImageList read FImageList write SetImageList;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TMatrixImage]);
end;

constructor TMatrixImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  Row := 1;
  Col := 1;
  PictureHeight := 32;
  PictureWidth := 32;
  Transparent := True;
end;

destructor TMatrixImage.Destroy;
begin
  ResizePicturesMatrix(0, 0);
  inherited Destroy;
end;

function TMatrixImage.DestRect(RowNum, ColNum: Integer): TRect;
var
  x, y: Integer;
begin
  x := ColNum * FPictureWidth;
  y := RowNum * FPictureHeight;
  if Stretch then
    Result := Bounds(x, y, FPictureWidth, FPictureHeight)
  else if Center then
    Result := Bounds(x + (FPictureWidth - FImageList.Width) div 2,
      y + (FPictureHeight - FImageList.Height) div 2,
        FImageList.Width, FImageList.Height)
  else
    Result := Bounds(x, y, FImageList.Width, FImageList.Height);
end;

procedure TMatrixImage.Paint;
var
  r, c: Integer;
  Save: Boolean;
begin
  if csDesigning in ComponentState then
    with inherited Canvas do
    begin
      Pen.Style := psDash;
      Brush.Style := bsClear;
      for r := 0 to FRow - 1 do
        for c := 0 to FCol - 1 do
          Rectangle(DestRect(r, c));
    end;
  Save := FDrawing;
  FDrawing := True;
  try
    (Parent as TWinControl).DoubleBuffered := True;
    with inherited Canvas do
      for r := 0 to FRow - 1 do
        for c := 0 to FCol - 1 do
          if FPictures[r, c] <> nil then
            StretchDraw(DestRect(r, c), FPictures[r, c].Graphic);
  finally
    FDrawing := Save;
  end;
end;

function TMatrixImage.DoPaletteChange(RowNum, ColNum: Integer): Boolean;
var
  ParentForm: TCustomForm;
  Tmp: TGraphic;
begin
  Result := False;
  if PictureExist(RowNum, ColNum) then
    Tmp := FPictures[RowNum, ColNum].Graphic
  else
    Tmp := nil;
  if Visible and (not (csLoading in ComponentState)) and (Tmp <> nil) and
    (Tmp.PaletteModified) then
  begin
    if (Tmp.Palette = 0) then
      Tmp.PaletteModified := False
    else
    begin
      ParentForm := GetParentForm(Self);
      if Assigned(ParentForm) and ParentForm.Active and Parentform.HandleAllocated then
      begin
        if FDrawing then
          ParentForm.Perform(wm_QueryNewPalette, 0, 0)
        else
          PostMessage(ParentForm.Handle, wm_QueryNewPalette, 0, 0);
        Result := True;
        Tmp.PaletteModified := False;
      end;
    end;
  end;
end;

procedure TMatrixImage.SetCenter(Value: Boolean);
begin
  if FCenter <> Value then
  begin
    FCenter := Value;
    PictureChanged(Self);
  end;
end;

procedure TMatrixImage.SetStretch(Value: Boolean);
begin
  if Value <> FStretch then
  begin
    FStretch := Value;
    PictureChanged(Self);
  end;
end;

procedure TMatrixImage.SetTransparent(Value: Boolean);
begin
  if Value <> FTransparent then
  begin
    FTransparent := Value;
    PictureChanged(Self);
  end;
end;

procedure TMatrixImage.PictureChanged(Sender: TObject);
var
  G: TGraphic;
  r, c: Integer;
begin
  for r := 0 to FRow - 1 do
    for c := 0 to FCol - 1 do
      if PictureExist(r, c) then
      begin
        G := FPictures[r, c].Graphic;
        if G <> nil then
        begin
            G.Transparent := FTransparent;
          if (not G.Transparent) and (Stretch or (G.Width >= Width)
            and (G.Height >= Height)) then
            ControlStyle := ControlStyle + [csOpaque]
          else
            ControlStyle := ControlStyle - [csOpaque];
          if DoPaletteChange(r, c) and FDrawing then Update;
        end
        else ControlStyle := ControlStyle - [csOpaque];
      end;
  if not FDrawing then Invalidate;
end;

function TMatrixImage.PictureExist(RowNum, ColNum: Integer): Boolean; //el
var
  Nr, Nc: Integer;
begin
  Result := False;
  if (RowNum < 0) or (ColNum < 0) then
    exit;
  Nr := Length(FPictures);
  if (Nr <= 0) or (RowNum >= Nr) then
    exit;
  Nc := Length(FPictures[0]);
  if (Nc <= 0) or (ColNum >= Nc) then
    exit;
  if FPictures[RowNum, ColNum] <> nil then
    Result := True;
end;

procedure TMatrixImage.SetRow(Value: Integer);
begin
  if Value <> Row then
  begin
    if Value < 1 then
      ShowMessage('Needed one row at last')
    else
    begin
      FRow := Value;
      ResizePicturesMatrix(FRow, FCol);
      PictureChanged(Self);
    end;
  end;
end;

procedure TMatrixImage.SetCol(Value: Integer);
begin
  if Value <> Col then
  begin
    if Value < 1 then
      ShowMessage('Needed one column at last')
    else
    begin
      FCol := Value;
      ResizePicturesMatrix(FRow, FCol);
      PictureChanged(Self);
    end;
  end;
end;

procedure TMatrixImage.SetPictureHeight(Value: Integer);
begin
  if Value <> PictureHeight then
  begin
    if Value < 1 then
      ShowMessage('PictureHeight has been > 0')
    else
    begin
      FPictureHeight := Value;
      ResizePicturesMatrix(FRow, FCol);
      PictureChanged(Self);
    end;
  end;
end;

procedure TMatrixImage.SetPictureWidth(Value: Integer);
begin
  if Value <> PictureWidth then
  begin
    if Value < 1 then
      ShowMessage('PictureWidth has been > 0')
    else
    begin
      FPictureWidth := Value;
      ResizePicturesMatrix(FRow, FCol);
      PictureChanged(Self);
    end;
  end;
end;

procedure TMatrixImage.ResizePicturesMatrix(NewRow, NewCol: Integer);
var
  Nr, Nc: Integer;
  r, c: Integer;
begin
  Nr := Length(FPictures);
  if Nr > 0 then
    Nc := Length(FPictures[0])
  else
    Nc := 0;

// ridimensiona l'oggetto
  SetBounds(Left, Top, NewCol * FPictureWidth, NewRow * FPictureHeight);

  if (NewRow = Nr) and (NewCol = Nc) then
    exit;

// cancella gli oggetti
  for r := 0 to Nr - 1 do
    for c := 0 to Nc - 1 do
    begin
      if FPictures[r, c] <> nil then
        FPictures[r, c].Free;
      FPicturesIndex[r, c] := -1;
    end;

// ridimensiona l'Matrix
  if (NewRow = 0) or (NewCol = 0) then
  begin
    SetLength(FPictures, 0, 0);
    SetLength(FPicturesIndex, 0, 0);
  end
  else
  begin
    SetLength(FPictures, NewRow, NewCol);
    SetLength(FPicturesIndex, NewRow, NewCol);
// crea gli oggetti
    for r := 0 to NewRow - 1 do
      for c := 0 to NewCol - 1 do
      begin
        FPictures[r, c] := TPicture.Create;
        FPictures[r, c].OnChange := PictureChanged;
        FPicturesIndex[r, c] := -1;
      end;
  end;
end;

procedure TMatrixImage.SetImageList;
begin
//  if FImageList <> Value then //11.10.04
  begin
    FImageList := Value;
    PictureChanged(Self);
  end;
end;

procedure TMatrixImage.AssignPictureIndex(RowNum, ColNum, Index: Integer);
begin
  if FImageList = nil then
    exit;
  if (Index < -1) or (Index > FImageList.Count - 1) then
    exit;
  if PictureExist(RowNum, ColNum) then
    if FPicturesIndex[RowNum, ColNum] <> Index then
    begin
      FPictures[RowNum, ColNum].Assign(nil);
      if Index > -1 then
        ImageList.GetBitmap(Index, FPictures[RowNum, ColNum].Bitmap);
      FPicturesIndex[RowNum, ColNum] := Index;
      PictureChanged(Self);
    end;
end;

procedure TMatrixImage.RemovePicture(RowNum, ColNum: Integer);
begin
  AssignPictureIndex(RowNum, ColNum, -1);
end;

end.
